import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

import org.json.JSONArray;
import org.json.JSONObject;

import com.nosoop.json.VDF;

public class SteamDirectoryHelper {
	private static Map<String, Path> libraryPathCache = null;
	
	private enum SystemType {
		WINDOWS, LINUX, MAC, OTHER
	}
	private static SystemType systemType = getSystemType();
	
	public static Path getSteamInstallPath() {
		String home = System.getProperty("user.home");
		if (systemType == SystemType.WINDOWS) {
			// 64 bit system?
			Path path = Paths.get("C:\\Program Files (x86)\\Steam\\steamapps");
			if (Files.exists(path)) return path.getParent();
			path = Paths.get("C:\\Program Files\\Steam\\steamapps");
			if (Files.exists(path)) return path.getParent();
			return null;
		} else if (systemType == SystemType.LINUX) {
			Path path = Paths.get(home + "/.local/share/Steam/steamapps");
			if (Files.exists(path)) return path.getParent();
			return null;
		} else if (systemType == SystemType.MAC) {
			Path path = Paths.get(home + "/Library/Application Support/Steam/steamapps");
			if (Files.exists(path)) return path.getParent();
			return null;
		}
		return null;
	}
	
	public static Path findGameInstallPath(Path steamInstallPath, String gameID) throws IOException {		
		Path libraryRootDir = getSteamLibraryPathsByGameID(steamInstallPath).get(gameID);
		
		if (libraryRootDir == null) return null;
		
		// now get the folder name our app has
		Path appManifest = libraryRootDir.resolve("steamapps/appmanifest_" + gameID + ".acf");
		String acfContent = new String(Files.readAllBytes(appManifest), StandardCharsets.UTF_8);
		JSONObject object = VDF.toJSONObject(acfContent, true);
		object = object.getJSONObject("AppState");
		String dirName = object.getString("installdir");
		
		return libraryRootDir.resolve("steamapps/common/" + dirName);
	}
	
	public static Path findWorkshopPath(Path steamInstallPath, String gameID) throws IOException {
		Path libraryRootDir = getSteamLibraryPathsByGameID(steamInstallPath).get(gameID);
		String workshopSubDir = "steamapps/workshop/content/" + gameID;
		if (libraryRootDir == null) {
			// try steam install dir
			Path p = steamInstallPath.resolve(workshopSubDir);
			if (Files.exists(p)) return p;
			return null;
		}
		return libraryRootDir.resolve(workshopSubDir);
	}
	
	public static synchronized Map<String, Path> getSteamLibraryPathsByGameID(Path steamInstallPath) throws IOException {
		if (libraryPathCache != null) return libraryPathCache;
		Path libraryFoldersFile = steamInstallPath.resolve("steamapps/libraryfolders.vdf");
		String vdfContent = new String(Files.readAllBytes(libraryFoldersFile), StandardCharsets.UTF_8);
		JSONObject object = VDF.toJSONObject(vdfContent, true);
		JSONArray folders = object.getJSONArray("libraryfolders");
		
		Path libraryRootDir = null;
		libraryPathCache = new TreeMap<>();
		for (int i = 0; i < folders.length(); ++i) {
			JSONObject folderEntry = folders.getJSONObject(i);
			libraryRootDir = Paths.get(folderEntry.getString("path"));
			JSONObject apps = folderEntry.getJSONObject("apps");
			
			Iterator<String> appIds = apps.keys();
			while(appIds.hasNext()) {
				String appId = appIds.next();
				libraryPathCache.put(appId, libraryRootDir);
			}
		}
		return libraryPathCache;
	}
	
	
	private static SystemType getSystemType() {
		String OS = System.getProperty("os.name").toLowerCase();
		if (OS.contains("win")) return SystemType.WINDOWS;
		if (OS.contains("mac")) return SystemType.MAC;
		if (OS.contains("nix") || OS.contains("nux") || OS.contains("aix")) return SystemType.LINUX;
		return SystemType.OTHER;
	}
}
