import java.awt.Color;
import java.awt.geom.AffineTransform;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;

import org.json.JSONArray;
import org.json.JSONObject;

public class WorkshopData {
	private String workshopID = "";
	private String title = "";
	private int width = 0;
	private int height = 0;
	private Color[][] imageData = null;
	private boolean[][] puzzleData = null;
	
	public WorkshopData(String id, JSONObject data) {
		this.workshopID = id;
		loadDataFromJSON(data);
	}
	
	public WorkshopData(String id, String title, int width, int height, Color[][] imageData, boolean[][] puzzleData) {
		this.workshopID = id;
		this.title = title;
		this.width = width;
		this.height = height;
		this.imageData = imageData;
		this.puzzleData = puzzleData;
	}
	
	private void loadDataFromJSON(JSONObject data) {
		this.title = data.getString("title");
		this.width = data.getInt("width");
		this.height = data.getInt("height");
		this.imageData = new Color[this.width][this.height];
		this.puzzleData = new boolean[this.width][this.height];
		JSONArray cells = data.getJSONArray("cells");
		
		// cells is a 2d array, saved by columns first
		for (int x = 0; x < cells.length(); ++x) {
			JSONArray columns = cells.getJSONArray(x);
			for (int y = 0; y < columns.length(); ++y) {
				JSONObject cellContent = columns.getJSONObject(y);
				this.puzzleData[x][y] = cellContent.getBoolean("puzzle");
				
				JSONObject color = cellContent.getJSONObject("color");
				this.imageData[x][y] = new Color(color.getFloat("r"), color.getFloat("g"), color.getFloat("b"), color.getFloat("a"));
			}
		}
	}
	
	public BufferedImage getSolutionImage(float zoom) {
		return ImageBuilder.createPixelImage(transformPuzzleDataToCellData(), zoom);
	}
	
	public BufferedImage getSolutionImageGrid() {
		return ImageBuilder.createGridImage(transformPuzzleDataToCellData());
	}
	
	
	
	public BufferedImage getClearImage(float zoom) {
		BufferedImage bufferedImage = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
		for (int x = 0; x < width; x++) {
		    for (int y = 0; y < height; y++) {
		        bufferedImage.setRGB(x, y, imageData[x][y].getRGB());
		    }
		}
		
		return zoomImage(bufferedImage, zoom);
	}
	
	private static BufferedImage zoomImage(BufferedImage image, float zoomRatio) {
		int w = image.getWidth();
		int h = image.getHeight();
		BufferedImage after = new BufferedImage((int)(w * zoomRatio), (int)(h * zoomRatio), BufferedImage.TYPE_INT_ARGB);
		AffineTransform at = new AffineTransform();
		at.scale(zoomRatio, zoomRatio);
		AffineTransformOp scaleOp = 
		   new AffineTransformOp(at, AffineTransformOp.TYPE_NEAREST_NEIGHBOR);
		after = scaleOp.filter(image, after);
		return after;
	}
	
	@Override
	public String toString() {
		return this.title + " (" + this.width + "*" + this.height + ")";
	}

	public String getWorkshopID() {
		return workshopID;
	}

	public String getTitle() {
		return title;
	}

	public int getWidth() {
		return width;
	}

	public int getHeight() {
		return height;
	}

	public Color[][] getImageData() {
		return imageData;
	}

	public boolean[][] getPuzzleData() {
		return puzzleData;
	}
	
	private CellType[][] transformPuzzleDataToCellData() {
		CellType[][] cellTypes = new CellType[width][height];
		for (int x = 0; x < width; x++) {
		    for (int y = 0; y < height; y++) {
		    	cellTypes[x][y] = puzzleData[x][y] ? CellType.MARKED_AS_FILLED : CellType.EMPTY;
		    }
		}
		return cellTypes;
	}
}
